import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

class ReportLaborCostDaily(Model):
    _name="clinic.report.labor.cost.daily"
    _string="Report Labor Cost Daily"
    _transient=True
    
    _fields={
        "date": fields.Date("Date"),
        "staff_type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        'department_id': fields.Many2One("clinic.department",'Department'),
    }

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date=defaults.get("date")
        if not date:
            date=time.strftime("%Y-%m-%d")
        staff_type=defaults.get("staff_type","doctor")
        staff_id=int(defaults.get('staff_id', "0"))
        department_id=int(defaults.get('department_id', "0"))
        res={
            'date': date,
            'staff_type': staff_type,
            'staff_id': staff_id and staff_id or None,
            'department_id': department_id and department_id or None,
        }
        print("res ", res)
        return res
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        comp=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date=defaults.get("date")
        staff_id=defaults.get("staff_id")
        staff_type=defaults.get("staff_type")
        dpt_id=defaults.get("department_id")
        dom=[]
        if ids:
            obj=self.browse(ids)[0]
            date=obj.date
            staff_id=obj.staff_id.id
            staff_type=obj.type
            dpt_id=obj.department_id.id
        dom.append(['date','>=',date])
        dom.append(['date','<=',date])
        # prevent to load more
        if not staff_id:
            print("not staff_id")
            return {}
        if staff_id:
            dom.append(['staff_id','=',staff_id])
        if staff_type:
            dom.append(['type','=',staff_type])
        if dpt_id:
            dom.append(['labor_cost_id.cycle_item_id.department_id','=',dpt_id])
        print('dom ', dom)
        lines=[]
        no=1
        for line in get_model("clinic.labor.cost.line").search_browse(dom):
            lcost=line.labor_cost_id
            citem=lcost.cycle_item_id
            cycle=citem.cycle_id
            staff=line.staff_id
            dpt=citem.department_id
            amt=line.amount or 0
            if staff_type=='doctor':
                for hdcase in citem.hd_cases:
                    doctor=hdcase.doctor_id
                    pt_type=hdcase.patient_type_id
                    pt=hdcase.patient_id
                    if staff.id==doctor.id:
                        lines.append({
                            'hd_case_number': hdcase.number or '',
                            'hd_case_id': hdcase.id or '',
                            'no': no,
                            'hn': pt.number,
                            'patient_id': pt.id,
                            'patient_name': pt.name or '',
                            'patient_type_name': pt_type.name or '',
                            'amount': amt,
                            'dpt_id': dpt.id,
                            'dpt_name': dpt.name or "",
                            'cycle_name': cycle.name,
                            'cycle_id': cycle.id,
                        })
                        no+=1
            elif staff_type=='nurse':
                print("nurse ")
                for hdcase in citem.hd_cases:
                    doctor=hdcase.doctor_id
                    pt_type=hdcase.patient_type_id
                    pt=hdcase.patient_id
                    lines.append({
                        'no': no,
                        'cycle_item_name': citem.name or '',
                        'cycle_item_id': citem.id,
                        'hn': pt.number,
                        'patient_id': pt.id,
                        'patient_type_name': pt_type.name or '',
                        'patient_name': pt.name or '',
                        'dpt_id': dpt.id,
                        'dpt_name': dpt.name or "",
                        'amount': 0,
                        'cycle_name': cycle.name,
                        'cycle_id': cycle.id,
                    })
                    no+=1
            else:
                continue
        data={
            'staff_type': staff_type,
            'date': date,
            'comp_name': comp.name or 0,
            'lines': lines,
        }
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['staff_id']=None
        return data

ReportLaborCostDaily.register()
