import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

class ReportLaborCostSummary(Model):
    _name="clinic.report.labor.cost.summary"
    _string="Report Labor Cost Summary"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        "staff_type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'level_id': fields.Many2One("clinic.staff.level","Level"),
        'categ_id': fields.Many2One("clinic.staff.categ","Category"),
        'only_value': fields.Boolean("Only Amount"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date_from=defaults.get("date", self._get_date_from())
        date_to=defaults.get("date", self._get_date_to())
        staff_type=defaults.get("staff_type","doctor")
        staff_id=int(defaults.get('staff_id', "0"))
        print('defaults ', defaults)
        res={
            'date': time.strftime("%Y-%m-%d"),
            'date_from': date_from,
            'date_to': date_to,
            'staff_type': staff_type,
            'staff_id': staff_id and staff_id or None,
            'only_value': True,
        }
        return res
    
    def get_report_data(self,ids,context={}):
        res=get_model("clinic.report.labor.cost.summary").default_get(context=context)
        date_from=res['date_from']
        date_to=res['date_to']
        staff_id=res['staff_id']
        staff_type=res['staff_type']
        only_value=res['only_value']
        level_id=None
        categ_id=None
        dom=[]
        if ids:
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            staff_id=obj.staff_id.id
            staff_type=obj.type
            level_id=obj.level_id.id
            categ_id=obj.categ_id.id
            only_value=obj.only_value
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        if staff_id:
            dom.append(['staff_id','=',staff_id])
        if staff_type:
            dom.append(['type','=',staff_type])
        if only_value:
            dom.append(['amount','!=',0])
        if categ_id:
            dom.append(['staff_id.categ_id','=',categ_id])
        staffs={}
        for line in get_model("clinic.labor.cost.line").search_browse(dom):
            lcost=line.labor_cost_id
            citem=lcost.cycle_item_id
            dpt=citem.department_id
            amt=line.amount or 0
            staff=line.staff_id
            categ_name=''
            categ_id=None
            categ=staff.categ_id
            if level_id and staff.level_id.id!=level_id:
                continue
            if categ:
                categ_id=categ.id
                categ_name=categ.name or ""
            if not staffs.get(staff.name):
                staffs[staff.name]={
                'number': staff.number or '',
                'staff_id': staff.id,
                'staff_type': staff.type,
                'staff_level': staff.level_id.name,
                'categ_name': categ_name,
                'categ_id': categ_id,
                dpt.name: {
                    'amt': 0,
                    },
                }
            if not staffs[staff.name].get(dpt.name):
                staffs[staff.name].update({
                        dpt.name: {
                            'amt': 0,
                        }})
            staffs[staff.name][dpt.name]['amt']+=amt
        lines=[]
        dpts=get_model("clinic.department").search_read([],['name'])
        dpts=sorted(dpts, key=lambda b: b['name'])
        no=1
        staff_types=set()
        snames=sorted(staffs.keys()) #sort by staff name
        for sname in snames:
            vals=staffs[sname]
            lvals={
                'no': no,
                'staff_name': sname,
                'staff_id': vals.get('staff_id'),
                'staff_type': vals.get('staff_type'),
                'staff_level': vals.get('staff_level'),
                'categ_name': vals.get('categ_name'),
                'categ_id': vals.get('categ_id'),
                'number': vals.get("number"),
            }
            total=0
            lvals['sub_lines']=[]
            for dpt in dpts:
                amt=0 #solve bug
                dname=dpt['name'] or ''
                vals2=vals.get(dname)
                if vals2:
                    amt=vals2.get("amt",0)
                    staff_type=vals2.get("type",'')
                    staff_types.update({staff_type})
                lvals['sub_lines'].append({'amt': amt,'dpt_id': dpt['id']})
                total+=amt
            lvals['total']=total
            lines.append(lvals)
            no+=1

        title=''
        for stype in list(staff_types):
            if stype=='doctor':
                title+='Doctor'
            elif stype=='nurse':
                title+='Nurse'
        title=' and'.join(t for t in title.split(" "))

        total_lines=[{'amt': 0} for dpt in dpts]
        for line in lines:
            i=0
            for sub_line in line['sub_lines']:
                amt=sub_line['amt'] or 0
                total_lines[i]['amt']+=amt
                i+=1
        total=0
        for tline in total_lines:
            amt=tline['amt'] or 0
            total+=amt

        total_lines.append({'amt': total})
        company_id=get_active_company()
        comp=get_model("company").browse(company_id)
        data={
            'title': title,
            'date_from': date_from,
            'date_to': date_to,
            'dpts': dpts,
            'comp_name': comp.name or 0,
            'comp_span': len(dpts),
            'lines': lines,
            'total_lines': total_lines,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['staff_id']=None
        return data

ReportLaborCostSummary.register()
